/***************************************************************************

    filename             : TRBNode.C
    description          : 
    created              : Sat Jul 19 2003
    version              : 1.0
    author               : Micha D. Niskin
    email                : mniskin@yahoo.com

    Copyright (C) 2003 Micha D. Niskin

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 ***************************************************************************/


#include <TClass.h>
#include <TMath.h>
#include "TRBNode.h"


ClassImp(TRBNode)

//-------------------------------------------------------------------------

TRBNode::TRBNode() : TObject()
{
   fKey1    = 0;
   fKey2    = 0;
   fd1      = 0;
   fd2      = 0;
   fData    = 0;
   kIsOwner = kFALSE;
}

//-------------------------------------------------------------------------

TRBNode::TRBNode(Double_t key1, Double_t key2, Double_t d1, Double_t d2,
                 TObject* data) : TObject()
{
   fKey1    = key1;
   fKey2    = key2;
   fd1      = d1;
   fd2      = d2;
   fData    = data;
   kIsOwner = kFALSE;
}

//-------------------------------------------------------------------------

   /*
    * Notice that the data object is only destroyed in this object's
    * destructor if SetIsOwner(kTRUE) has been called.
    */

TRBNode::~TRBNode()
{
   if (IsOwner())
      delete fData;
}

//-------------------------------------------------------------------------

   /*
    * Since TRBNodes are sortable objects, there must be an IsEqual() method
    */

Bool_t TRBNode::IsEqual(const TObject* obj) const
{
   if (IsA() != obj->IsA())
      Error("IsEqual","Attempt to compare with a %s", obj->IsA()->GetName());

   return (!Compare(obj));
}

//-------------------------------------------------------------------------

   /*
    * Compares this object with another TRBNode, TRBNodes being sortable 
    * objects. The two keys belonging to each object are treated as halves
    * of a double Double_t, i.e., 
    * (key1,key2) > (key3,key4) iff strcat(key1,key2) > strcat(key3,key4).
    * There is a little fuzzyness allowed in that differences are only
    * considered to within fd1 in key1 and fd2 in key2. This means that,
    * for example, (5,1) == (4,4) if fd1 >= 1 and fd2 >= 3.
    */
    
Int_t TRBNode::Compare(const TObject* obj) const
{
   if (IsA() != obj->IsA())
      Error("Compare","Attempt to compare with a %s", obj->IsA()->GetName());

   Double_t objkey1 = ((TRBNode*) obj)->fKey1;
   Double_t objkey2 = ((TRBNode*) obj)->fKey2;

   if ((TMath::Abs(fKey1 - objkey1) <= fd1/2) &&
       (TMath::Abs(fKey2 - objkey2) <= fd2/2))
      return 0;
   return ((fKey1 < objkey1 - fd1/2) || 
           ((fKey1 <= objkey1 + fd1/2) && (fKey2 < objkey2 - fd2/2))) ? -1 : 1;
}
