/***************************************************************************

    filename             : TRBTree.h
    description          : 
    created              : Sun Jul 20 2003
    version              : 1.0
    author               : Micha D. Niskin
    email                : mniskin@yahoo.com

    Copyright (C) 2003 Micha D. Niskin

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 ***************************************************************************/


#ifndef FIU_TRBTREE
#define FIU_TRBTREE


#include <TObject.h>
#include <TClass.h>
#include <TInterpreter.h>
#include <iostream>
#include "TRBNode.h"


class TRBTree : public TObject {

public:

   // public methods
   TRBTree();
   TRBTree(TObject* obj);
   ~TRBTree();

   void              Add(TObject* obj);
   void              Add(TRBTree* tree);
   void              Clear(Option_t* option = "");
   virtual Int_t     Compare(const TObject* obj) const;
   virtual void      Delete(Option_t* option = "");
   virtual TObject*  FindObject(const TObject* obj) const;
   TRBTree*          FindSubtreeContaining(TObject* obj);
   Int_t             GetDepth() const;
   TRBTree*          GetRoot();
   Int_t             GetSize() const;
   virtual Bool_t    IsEqual(const TObject* obj) const;
   Bool_t            IsOwner() const    { return kIsOwner; }
   virtual Bool_t    IsSortable() const { return kTRUE;    }
   TObject*          Remove(TObject* obj);
   void              SetOwner(Bool_t enable=kTRUE);
   virtual Int_t     Write(const char* name="0", 
                           Int_t option=TObject::kSingleKey, Int_t bsize=0);

   // Methods which are public, but generally not recommended. These are
   // here due to the recursive nature of tree operations and the necessity
   // of making these methods available for calling by other TRBTree objects
   // during recursion. All but the last method in this group can cause the
   // tree to become unbalanced, resulting in erratic behavior and segfaults.
   void              Balance();
   void              ColorFlip();
   void              LeftRotate();
   void              RightRotate();
   void              DoubleLeftRotate();
   void              DoubleRightRotate();
   Bool_t            IsLeftChild();

   // public members
   Bool_t            kIsBlack;   // Used by balancing algorithm
   TObject*          fData;      // Object stored in tree
   TRBTree*          fParent;    // Parent node
   TRBTree*          fLeft;      // Left child node
   TRBTree*          fRight;     // Right child node

private:

   //private members
   Bool_t            kIsOwner;   // TRUE => stored objects are to be deleted

   ClassDef(TRBTree,1)

};

#endif
