/***************************************************************************

    filename             : dat2root.C
    description          : 
    created              : Mon Jun 30 2003
    version              : 1.0
    author               : Micha D. Niskin
    email                : mniskin@yahoo.com

    Copyright (C) 2003 Micha D. Niskin

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 ***************************************************************************/


#include <iostream>
#include <fstream>
#include <string>
#include <TString.h>
#include <TDirectory.h>
#include <TFile.h>
#include <TDatime.h>
#include <snprintf.h>
#include "THaRfn.h"


using namespace std;


void Usage();

//-------------------------------------------------------------------------

int main(int argc, char **argv)
{
   string dir="", name="rfn", title="rfn", outfile="rfn.root";
    
   int i=1;
   while (i < argc) {
      if (!strcmp(argv[i],"--")) {
         dir = argv[i];
         break;
      } else if (!strcmp(argv[i],"-h")) {
         Usage();
         return 1;
      } else if (!strcmp(argv[i],"-n")) {
         if (i + 1 < argc) {
            name = argv[i + 1];
            i += 2;
         } else {
            cerr << "Bad argument: '" << argv[i] << "' not recognized\n";
            Usage();
            return 1;
         }
      } else if (!strcmp(argv[i],"-t")) {
         if (i + 1 < argc) {
            title = argv[i + 1];
            i += 2;
         } else {
            cerr << "Bad argument: '" << argv[i] << "' not recognized\n";
            Usage();
            return 1;
         }
      } else if (!strcmp(argv[i],"-o")) {
         if (i + 1 < argc) {
            outfile = argv[i + 1];
            i += 2;
         } else {
            cerr << "Bad argument: '" << argv[i] << "' not recognized\n";
            Usage();
            return 1;
         }
      } else if (*argv[i] == '-') {
         cerr << "Bad argument: '" << argv[i] << "' not recognized\n";
         Usage();
         return 1;
      } else {
         dir = argv[i];
         i += 1;
      }
   }
         
   if (!strcmp(dir.data(), "")) {
      Usage();
      return 1;
   }

   THaRfn *rf = new THaRfn;
   rf->LoadDefs(dir.data());
   rf->SetNameTitle(name.data(), title.data());

   TFile f(outfile.data(), "UPDATE");
   rf->Write();
   f.Close();

   return(0);
}

//-------------------------------------------------------------------------

void Usage()
{
   cout << "\nUsage: dat2root [options] <dat directory>\n";
   cout << "       dat2root -h\n\n";
   cout << "Options:\n";
   cout << "   --                Indicates end of options (useful when the dat directory\n";
   cout << "                     starts with the '-' character).\n";
   cout << "   -h                Display this help info.\n";
   cout << "   -n <rfn name>     Specifies name of rfn object. Default name is 'rfn'.\n";
   cout << "   -o <outfile>      Specifies output file. Default name is 'rfn.root'.\n";
   cout << "   -t <rfn title>    Specifies title of rfn object. Default title is 'rfn'.\n\n";
   cout << "Example: dat2root -o rfn1997012.root -n rfn1 -t 'left arm' ./data\n\n";
   cout << "New THaRfn object will be appended to output file if it already exists.\n\n";
}
