#define h1_cxx
#include "h1.h"
#include "TH2.h"
#include "TStyle.h"
#include "TCanvas.h"

void h1::Loop()
{
//   In a ROOT session, you can do:
//      Root > .L h1.C
//      Root > h1 t
//      Root > t.GetEntry(12); // Fill t data members with entry number 12
//      Root > t.Show();       // Show values of entry 12
//      Root > t.Show(16);     // Read and show values of entry 16
//      Root > t.Loop();       // Loop on all entries
//

//     This is the loop skeleton where:
//    jentry is the global entry number in the chain
//    ientry is the entry number in the current Tree
//  Note that the argument to GetEntry must be:
//    jentry for TChain::GetEntry
//    ientry for TTree::GetEntry and TBranch::GetEntry
//
//       To read only selected branches, Insert statements like:
// METHOD1:
//    fChain->SetBranchStatus("*",0);  // disable all branches
//    fChain->SetBranchStatus("branchname",1);  // activate branchname
// METHOD2: replace line
//    fChain->GetEntry(jentry);       //read all branches
//by  b_branchname->GetEntry(ientry); //read only this branch
   if (fChain == 0) return;

   Int_t nentries = Int_t(fChain->GetEntriesFast());

   Float_t y_p, dp_p, th_p, ph_p, aedv, rfn_p;
   Float_t y_e, dp_e, th_e, ph_e, rfn_e;
   Float_t rfn_lc, rfn_ln, rfn_rc, rfn_rn;

   TH1F *hist1 = new TH1F("Root Rfn","Combined",100,0,0.04);
   TH1F *hist2 = new TH1F("Reference Rfn","Reference Rfn",100,0,0.04);
   TH1F *hist3 = new TH1F("Difference","Difference",100,-0.03,0.03);

   TCanvas *c1 = new TCanvas("c1","hist output",400,400);
   TCanvas *c3 = new TCanvas("c3","hist output",400,400);

   fChain->SetBranchAddress("Y_p_tg",&y_p);
   fChain->SetBranchAddress("Dp_p_tg",&dp_p);
   fChain->SetBranchAddress("Th_p_tg",&th_p);
   fChain->SetBranchAddress("Ph_p_tg",&ph_p);
   fChain->SetBranchAddress("Aedv",&aedv);

   fChain->SetBranchAddress("Y_e_tg",&y_e);
   fChain->SetBranchAddress("Dp_e_tg",&dp_e);
   fChain->SetBranchAddress("Th_e_tg",&th_e);
   fChain->SetBranchAddress("Ph_e_tg",&ph_e);

   fChain->Branch("rfn_rc",&rfn_rc,"rfn_rc/D");

   if (!TClassTable::GetDict("THaRfn"))
      gROOT->ProcessLine(".L libRfn2.so");

   TFile f("output/rfn.root","READ");

   THaRfn *rc;

   if (!(rc = (THaRfn*) f.Get("left-c")))
      cout << "NO RFN\n";

   TStopwatch s;

   Int_t step = 0;

   Int_t nbytes = 0, nb = 0;
   for (Int_t jentry=0; jentry<nentries;jentry++) {
      Int_t ientry = LoadTree(jentry);
      if (ientry < 0) break;
      nb = fChain->GetEntry(jentry);   nbytes += nb;
      // if (Cut(ientry) < 0) continue;
      
      Double_t y  = (Double_t) y_p/100;
      Double_t dp = (Double_t) dp_p/100;
      Double_t th = (Double_t) th_p/1000;
      Double_t ph = (Double_t) ph_p/1000;

      rfn_rc = (Float_t) rc->Eval(y, dp, th, ph);

      Float_t diff = rfn_rc - aedv;
      
      hist1->Fill(rfn_rc);
      hist2->Fill(aedv);
      hist3->Fill(diff);

      //fChain->Fill();
   }
   s.Stop();

   cout << "got " << jentry << " entries\n";
   s.Print();

   c1->cd();
   hist1->Draw();
   hist2->Draw("SAMES");
   c3->cd();
   hist3->Fit("gaus");
   hist3->Draw();
}
