/***************************************************************************

    filename             : TRfn.h
    description          : 
    created              : Tue Jul 15 2003
    version              : 1.0
    author               : Micha D. Niskin
    email                : mniskin@yahoo.com

    Copyright (C) 2003 Micha D. Niskin

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

 ***************************************************************************/


#ifndef FIU_TRFN
#define FIU_TRFN

#include <TObject.h>
#include <TMath.h>


class TRfn : public TObject {

public:
   enum  ERfnOp_t { kNONE=0, kAND, kOR, kPRIMITIVE };
   enum  ERfnErr_t { kBAD_RFN=-1000 };

   TRfn();
   TRfn(Double_t, Double_t, Double_t);
   TRfn(const TRfn&);
   ~TRfn();
   ERfnOp_t    GetOp()     const { return fOp;                          }
   Double_t    GetA()      const { return fA;                           }
   Double_t    GetB()      const { return fB;                           }
   Double_t    GetC()      const { return fC;                           }
   TRfn*       GetChild1() const { return (fChild1) ? fChild1 : 0;      }
   TRfn*       GetChild2() const { return (fChild2) ? fChild2 : 0;      }
   Double_t    GetDR_DTh() const { return fDR_DTh;                      }
   Double_t    GetDR_DPh() const { return fDR_DPh;                      }
   Double_t    GetErr(Double_t dth, Double_t dph) const;
   Double_t    Eval(Double_t th, Double_t ph) { return (*this)(th, ph); }
   Bool_t      IsNegated() const { return fIsNegated;                   }
   Bool_t      IsValid()   const { return fIsValid;                     }
   void        SetIsNegated(Bool_t state) { fIsNegated = state;         }
   void        SetIsValid(Bool_t state) { fIsValid = state;             }
   TRfn        operator= (TRfn);
   TRfn        operator! ();
   TRfn        operator& (TRfn);
   TRfn        operator| (TRfn);
   void        operator&= (TRfn);
   void        operator|= (TRfn);
   Double_t    operator() (Double_t th, Double_t ph);

protected:
   virtual Double_t Primitive(Double_t th, Double_t ph);
   virtual Double_t And(Double_t th, Double_t ph);
   virtual Double_t Or(Double_t th, Double_t ph);
   virtual Double_t DR_DTh(Double_t th, Double_t ph) { return fDR_DTh; }
   virtual Double_t DR_DPh(Double_t th, Double_t ph) { return fDR_DPh; }
   
private:
   Double_t    fA;
   Double_t    fB;
   Double_t    fC;
   Double_t    fDR_DTh;
   Double_t    fDR_DPh;
   ERfnOp_t    fOp;
   TRfn*       fChild1;
   TRfn*       fChild2;
   Bool_t      fIsValid;
   Bool_t      fIsNegated;

   ClassDef(TRfn,1);

};

//-------------------------------------------------------------------------

inline Double_t TRfn::GetErr(Double_t dth, Double_t dph) const
{
   Double_t drdth=GetDR_DTh(), drdph=GetDR_DPh();

   return( TMath::Sqrt(drdth*dth*drdth*dth + drdph*dph*drdph*dph) );
}

#endif
